@@ -3,7 +3,7 @@
 //      $Workfile:   imb_api.h  $
 //      $Revision:   1.0  $
 //      $Modtime:   Jul 22 2002 16:40:32  $
-//      $Author:   arcress@users.sf.net  $
+//      $Author:   arcress at users.sf.net  $
 // 
 //  Combined include files needed for imbapi.c
 //
@@ -11,6 +11,7 @@
 /*----------------------------------------------------------------------* 
 The BSD License 
 Copyright (c) 2002, Intel Corporation
+Copyright (c) 2009 Kontron America, Inc.
 All rights reserved.
 Redistribution and use in source and binary forms, with or without 
 modification, are permitted provided that the following conditions are met:
@@ -42,13 +43,18 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #define TRUE    1
 #define FALSE   0
 
-#ifndef WIN32   
-/* WIN32 defines this in stdio.h */
+#if defined(WIN32) | defined(BSD) | defined(DOS)
+/* WIN32 defines wchar_t in stdio.h */
+/* BSD defines wchar_t in stdlib.h */
+/* DOS defines wchar_t in stdlib.h */
+#else 
+ // defined(LINUX) | defined(SOLARIS)
 #ifndef _WCHAR_T
 #define _WCHAR_T
 typedef long    wchar_t;
 #endif
 #endif
+
 #define far
 #define near
 #define FAR                 far
@@ -79,7 +85,9 @@ typedef CONST void far      *LPCVOID;
 typedef int                 INT;
 typedef unsigned int        UINT;
 typedef unsigned int        *PUINT;
+#ifndef NTSTATUS
 typedef DWORD NTSTATUS;
+#endif
 /*
   File structures
 */
@@ -147,13 +155,15 @@ typedef UNICODE_STRING *PUNICODE_STRING;
 #define OUT	/* */
 #define OPTIONAL	/* */
 
-#ifndef WIN32
+#ifdef WIN32
+#define HandleType  HANDLE
+#else
+#define HandleType  long
 #define FIELD_OFFSET(type, field)    ((LONG)&(((type *)0)->field))
 #define UNREFERENCED_PARAMETER(x)
-typedef	int HANDLE;
-#define	INVALID_HANDLE_VALUE	((HANDLE)-1)
+#define	INVALID_HANDLE_VALUE	((HandleType)-1)
 #endif
-typedef	HANDLE	*PHANDLE;
+typedef	HandleType *PHANDLE;
 /*
  Define the method codes for how buffers are passed for I/O and FS controls
 */
@@ -169,7 +179,7 @@ typedef	HANDLE	*PHANDLE;
   These are the generic rights.
 */
 #define    MAX_PATH        260
-#define	GetLastError()	(NTstatus.Status)
+// #define	GetLastError()	(NTstatus.Status)
 /*
  Macro definition for defining IOCTL and FSCTL function control codes.  Note
  that function codes 0-2047 are reserved for Microsoft Corporation, and
@@ -188,10 +198,13 @@ typedef	HANDLE	*PHANDLE;
 #define CTL_CODE(DeviceType, Function, Method, Access)\
 		_IO(DeviceType & 0x00FF, Function & 0x00FF)
 #else
+/* WIN32 may have this defined in winsdk\include\winioctl.h */
+#ifndef CTL_CODE
 #define CTL_CODE( DeviceType, Function, Method, Access ) ((ULONG)(	\
     ((ULONG)(DeviceType) << 16) | ((ULONG)(Access) << 14) | ((ULONG)(Function) << 2) | ((ULONG)Method) \
 ))
 #endif
+#endif
 #endif /*_WINDEFS_H */
 /*----------------------------------------------------------------------*/
 #ifndef	_SMI_H
@@ -238,10 +251,10 @@ typedef struct _IO_STATUS_BLOCK {
 #define STATUS_UNSUCCESSFUL              (0xC0000001U)
 #define STATUS_DEVICE_BUSY               (0x80000011U)
 #ifndef WIN32
-#define STATUS_PENDING                   (0x00000103U)
 // see <win2000ddk>\inc\winnt.h(1171)
-#endif
+#define STATUS_PENDING                   (0x00000103U)
 #define STATUS_INVALID_PARAMETER         (0xC000000DU)
+#endif
 #define STATUS_INVALID_DEVICE_REQUEST    (0xC0000010U)
 #define STATUS_BUFFER_TOO_SMALL          (0xC0000023U)
 #define STATUS_FILE_CLOSED               (0xC0000128U)
@@ -285,8 +298,9 @@ typedef struct {
 	BYTE data[1];
 } ImbPacket;
 #define MIN_IMB_PACKET_SIZE	7 	
-#define MAX_IMB_PACKET_SIZE	33
+#define MAX_IMB_PACKET_SIZE	57   /*was 33, now 64-7=57*/
 // #define MAX_IMB_PACKET_SIZE	135  (if using flashupdt large packets)
+#define	MAX_RBUFFER_SIZE	64   /*no longer used*/
 /*
  * This is the standard IMB response format where the first byte of
  * IMB packet data is interpreted as a command completion code.
@@ -326,7 +340,7 @@ typedef struct {
 	BYTE netFn;
 	BYTE rsLun;	
 	BYTE dataLength;
-	BYTE data[1];	
+	BYTE data[16];	
 } ImbRequest;
 typedef struct {
    UINT32	flags;			/* request flags*/
@@ -335,6 +349,7 @@ typedef struct {
    ImbRequest	req;			/* message buffer*/
 } ImbRequestBuffer;
 #define MIN_IMB_REQ_BUF_SIZE	13	/* a buffer without any request data*/
+#define MAX_IMB_REQ_SIZE	(MIN_IMB_RESP_BUF_SIZE + MAX_IMB_PACKET_SIZE)
 /************************
  *  ImbResponseBuffer
  ************************/
@@ -352,7 +367,7 @@ typedef struct {
 	BYTE       data[1];	
 } ImbResponseBuffer;
 #define MIN_IMB_RESP_BUF_SIZE	1	
-#define MAX_IMB_RESP_SIZE		(MIN_IMB_RESP_BUF_SIZE + MAX_IMB_RESPONSE_SIZE)
+#define MAX_IMB_RESP_SIZE	(MIN_IMB_RESP_BUF_SIZE + MAX_IMB_RESPONSE_SIZE)
 #pragma pack()
 
 /*
@@ -388,15 +403,15 @@ typedef struct {
 */
 #define FILE_DEVICE_IMB			0x00008010
 #define IOCTL_IMB_BASE			0x00000880
-#define IOCTL_IMB_SEND_MESSAGE		CTL_CODE(FILE_DEVICE_IMB, IOCTL_IMB_BASE + 2, METHOD_BUFFERED, FILE_ANY_ACCESS)
-#define IOCTL_IMB_GET_ASYNC_MSG		CTL_CODE(FILE_DEVICE_IMB, IOCTL_IMB_BASE + 8, METHOD_BUFFERED, FILE_ANY_ACCESS)
-#define IOCTL_IMB_MAP_MEMORY		CTL_CODE(FILE_DEVICE_IMB, IOCTL_IMB_BASE + 14,METHOD_BUFFERED, FILE_ANY_ACCESS)
-#define IOCTL_IMB_UNMAP_MEMORY		CTL_CODE(FILE_DEVICE_IMB, IOCTL_IMB_BASE + 16,METHOD_BUFFERED, FILE_ANY_ACCESS)
-#define IOCTL_IMB_SHUTDOWN_CODE		CTL_CODE(FILE_DEVICE_IMB, IOCTL_IMB_BASE + 18,METHOD_BUFFERED, FILE_ANY_ACCESS)
-#define IOCTL_IMB_REGISTER_ASYNC_OBJ	CTL_CODE(FILE_DEVICE_IMB, IOCTL_IMB_BASE + 24,METHOD_BUFFERED, FILE_ANY_ACCESS)
-#define IOCTL_IMB_DEREGISTER_ASYNC_OBJ	CTL_CODE(FILE_DEVICE_IMB, IOCTL_IMB_BASE + 26,METHOD_BUFFERED, FILE_ANY_ACCESS)
-#define IOCTL_IMB_CHECK_EVENT		CTL_CODE(FILE_DEVICE_IMB, IOCTL_IMB_BASE + 28,METHOD_BUFFERED, FILE_ANY_ACCESS)
-#define IOCTL_IMB_POLL_ASYNC		CTL_CODE(FILE_DEVICE_IMB, IOCTL_IMB_BASE + 20,METHOD_BUFFERED, FILE_ANY_ACCESS)
+#define IOCTL_IMB_SEND_MESSAGE		CTL_CODE(FILE_DEVICE_IMB, (IOCTL_IMB_BASE + 2), METHOD_BUFFERED, FILE_ANY_ACCESS)
+#define IOCTL_IMB_GET_ASYNC_MSG		CTL_CODE(FILE_DEVICE_IMB, (IOCTL_IMB_BASE + 8), METHOD_BUFFERED, FILE_ANY_ACCESS)
+#define IOCTL_IMB_MAP_MEMORY		CTL_CODE(FILE_DEVICE_IMB, (IOCTL_IMB_BASE + 14),METHOD_BUFFERED, FILE_ANY_ACCESS)
+#define IOCTL_IMB_UNMAP_MEMORY		CTL_CODE(FILE_DEVICE_IMB, (IOCTL_IMB_BASE + 16),METHOD_BUFFERED, FILE_ANY_ACCESS)
+#define IOCTL_IMB_SHUTDOWN_CODE		CTL_CODE(FILE_DEVICE_IMB, (IOCTL_IMB_BASE + 18),METHOD_BUFFERED, FILE_ANY_ACCESS)
+#define IOCTL_IMB_REGISTER_ASYNC_OBJ	CTL_CODE(FILE_DEVICE_IMB, (IOCTL_IMB_BASE + 24),METHOD_BUFFERED, FILE_ANY_ACCESS)
+#define IOCTL_IMB_DEREGISTER_ASYNC_OBJ	CTL_CODE(FILE_DEVICE_IMB, (IOCTL_IMB_BASE + 26),METHOD_BUFFERED, FILE_ANY_ACCESS)
+#define IOCTL_IMB_CHECK_EVENT		CTL_CODE(FILE_DEVICE_IMB, (IOCTL_IMB_BASE + 28),METHOD_BUFFERED, FILE_ANY_ACCESS)
+#define IOCTL_IMB_POLL_ASYNC		CTL_CODE(FILE_DEVICE_IMB, (IOCTL_IMB_BASE + 20),METHOD_BUFFERED, FILE_ANY_ACCESS)
 #endif /* IMB_IF__ */
 /*----------------------------------------------------------------------*/
 /*  No asynchronous messages available */
@@ -453,18 +468,23 @@ typedef struct {
 #ifndef IMBAPI_H__
 #define IMBAPI_H__
 #include <sys/types.h>
+#if defined(MACOS)
+/* has caddr_t */
+#else
 #ifndef _SYS_TYPES_H
 #ifndef _CADDR_T
 #define _CADDR_T
   typedef char *        caddr_t;
 #endif
 #endif
+#endif
 
 #define	WRITE_READ_I2C		0x52
 #define	WRITE_EMP_BUFFER	0x7a
 #define	GET_DEVICE_ID		0x1
 #define	CMD_GET_DEVICE_ID	0x01
 #define SEND_MESSAGE		0x34
+#define GET_MESSAGE		0x33
 #define BMC_SA			0x20
 #define BMC_LUN			0
 #define APP_NETFN		0x06
@@ -566,11 +586,6 @@ typedef struct
  * This is the generic IMB packet format, the final checksum cant be
  * represented in this structure and will show up as the last data byte
  */
-/*
- #define MIN_IMB_PACKET_SIZE	7
- #define MAX_IMB_PACKET_SIZE	33
-*/
-#define	MAX_RBUFFER_SIZE	64
 /*#endif // IMB_API */
 
 /****************************** 
@@ -684,10 +699,10 @@ SendTimedLanMessageResponse_Ex (
 	BYTE	  channelNumber
 	);
 ACCESN_STATUS
-IsAsyncMessageAvailable (unsigned int   eventId	);
+IsAsyncMessageAvailable (HandleType   eventId);
 ACCESN_STATUS
-RegisterForImbAsyncMessageNotification (unsigned int *handleId);
+RegisterForImbAsyncMessageNotification (HandleType *handleId);
 ACCESN_STATUS
-UnRegisterForImbAsyncMessageNotification (unsigned int handleId,int iFlag);
+UnRegisterForImbAsyncMessageNotification (HandleType handleId,int iFlag);
 BYTE	GetIpmiVersion(void);
 #endif /* IMBAPI_H__ */
