@@ -22,6 +22,7 @@
 /*----------------------------------------------------------------------* 
 The BSD License 
 Copyright (c) 2002-2006, Intel Corporation
+Copyright (c) 2009 Kontron America, Inc.
 All rights reserved.
 Redistribution and use in source and binary forms, with or without 
 modification, are permitted provided that the following conditions are met:
@@ -45,7 +46,14 @@ ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *----------------------------------------------------------------------*/
 
+#if defined(SOLARIS) || defined(BSD)
+#undef IMB_API 
+#else
+/* Ok for WIN32, LINUX, SCO_UW, etc., but not Solaris */
 #define IMB_API 1
+#endif
+
+#ifdef IMB_API
 
 #ifdef WIN32
 #define NO_MACRO_ARGS  1
@@ -64,6 +72,7 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include <stdlib.h>
 #include <fcntl.h>
 #include <string.h>
+#include <errno.h>
 #endif
 #include "imb_api.h"
 
@@ -77,10 +86,7 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #endif
 
 /*Just to make the DEBUG code cleaner.*/
-#ifdef NO_MACRO_ARGS
-#define DEBUG(format, args)  /*not_debug*/
-#else
-#ifdef LINUX_DEBUG
+#if defined(DBG_IPMI) || defined(LINUX_DEBUG) 
 #include <stdarg.h>
 static void dbgmsg( char *pattn, ... )
 {
@@ -88,12 +94,17 @@ static void dbgmsg( char *pattn, ... )
     FILE *fdlog;
 
     fdlog = fopen( "imbdbg.log", "a+" );
-    if (fdlog == NULL) fdlog = stderr;
+    if (fdlog == NULL) fdlog = stdout;
     va_start( arglist, pattn );
     vfprintf( fdlog, pattn, arglist );
     va_end( arglist );
     fclose(fdlog);
 }
+#endif
+#ifdef NO_MACRO_ARGS
+#define DEBUG(format, args)  /*not_debug*/
+#else
+#ifdef LINUX_DEBUG
 #define DEBUG(format, args...) dbgmsg(format, ##args)
 // #define DEBUG(format, args...) printf(format, ##args)
 #else
@@ -102,13 +113,25 @@ static void dbgmsg( char *pattn, ... )
 #endif
 
 #define IMB_OPEN_TIMEOUT      400
-#define IMB_MAX_RETRIES       3
-#define IMB_DEFAULT_TIMEOUT   (1000)   /*timeout for SendTimedImb in sec*/
+#define IMB_MAX_RETRIES       2
+#define IMB_DEFAULT_TIMEOUT   (1000)   /*timeout for SendTimedImb in msec*/
 #define ERR_NO_DRV         -16 /*cannot open IPMI driver*/
 
+#define LOCAL_IPMB  1   /*instead of METACOMMAND_IPMB*/
+
 int ipmi_timeout_ia = IMB_DEFAULT_TIMEOUT;  /* in msec */
+#ifdef METACOMMAND_IPMB   /*++++ not used*/
+extern int ipmi_cmd_ipmb(BYTE cmd, BYTE netfn, BYTE sa, BYTE bus, BYTE lun,
+                BYTE *pdata, int sdata, BYTE *presp,
+                int *sresp, BYTE *pcc, char fdebugcmd); /*ipmilan.c*/
+#endif
+#ifdef METACOMMAND
+extern FILE *fpdbg;
+extern FILE *fperr;
+#else
 static FILE *fpdbg = NULL;
 static FILE *fperr = NULL;
+#endif
 
 /* uncomment out the #define below or use -DLINUX_DEBUG_MAX in the makefile 
 // if you want a dump of the memory to debug mmap system call in
@@ -125,7 +148,7 @@ static FILE *fperr = NULL;
 // pre-processor directives to separate os-specific code (pai 11/21) */
 
 static int  IpmiVersion;
-static HANDLE  AsyncEventHandle = 0;
+static HandleType AsyncEventHandle = 0;
 // static void *  AsyncEventObject = 0; 
 
 /*////////////////////////////////////////////////////////////////////////////
@@ -140,8 +163,8 @@ static DWORD ioctl_sendmsg = IOCTL_IMB_SEND_MESSAGE;
 #endif
 
 static IO_STATUS_BLOCK NTstatus; /*dummy place holder. See deviceiocontrol. */
-static HANDLE   hDevice1; /*used in open_imb() */
-static HANDLE   hDevice;  /*used to call DeviceIoControl()*/
+static HandleType hDevice1; /*used in open_imb() */
+static HandleType hDevice;  /*used to call DeviceIoControl()*/
 /*mutex_t deviceMutex; */
 #ifdef LINUX_DEBUG 
 static char fdebug = 1;
@@ -169,7 +192,15 @@ static char fdebug = 0;
 
 int open_imb(int fskipcmd);
 
+void set_fps(void)
+{
+   if (fpdbg == NULL) fpdbg = stdout;
+   if (fperr == NULL) fperr = stdout;
+}
+
 #ifndef WIN32
+DWORD GetLastError(void) { return(NTstatus.Status); }  
+
 /*///////////////////////////////////////////////////////////////////////////
 // DeviceIoControl 
 ///////////////////////////////////////////////////////////////////////////// */
@@ -184,7 +215,7 @@ int open_imb(int fskipcmd);
 *F*/
 static BOOL
 DeviceIoControl(
-	HANDLE 			dummy_hDevice, /* handle of device */
+	HandleType		dummy_hDevice, /* handle of device */
 	DWORD 			dwIoControlCode, /* control code of operation to perform*/
 	LPVOID 			lpvInBuffer, /* address of buffer for input data */
 	DWORD 			cbInBuffer, /* size of input buffer */
@@ -202,6 +233,9 @@ DeviceIoControl(
 			hDevice1,hDevice,dwIoControlCode);
   	rc = open_imb(1); /*open if needed, set fskipcmd to avoid recursion*/
   	if (rc == 0) {
+#ifdef DBG_IPMI
+  	  dbgmsg("DeviceIoControl: open_imb failed %d\n", rc);
+#endif
     	  return FALSE;
     	  }
 
@@ -264,6 +298,10 @@ DeviceIoControl(
 #ifndef NO_MACRO_ARGS
  	  	DEBUG("%s %s: ioctl cmd = 0x%x failed %d \n", 
 			__FILE__,__FUNCTION__,dwIoControlCode,ioctl_status);
+#endif
+#ifdef DBG_IPMI
+ 	  	dbgmsg("DeviceIoControl: ioctl cmd = 0x%x failed %d \n", 
+			dwIoControlCode,ioctl_status);
 #endif
 		/*      mutex_unlock(&deviceMutex); */
     		return FALSE;
@@ -295,6 +333,180 @@ DeviceIoControl(
 }
 #endif
 
+static void _dump_buf(char *tag, BYTE *pbuf, int sz, char fshowascii)
+{
+   BYTE line[17];
+   BYTE a;
+   int i, j;
+   char *stag;
+   FILE *fpdbg1;
+
+#ifdef DBG_IPMI
+   fpdbg1 = fopen( "imbdbg.log", "a+" );
+   if (fpdbg1 == NULL) fpdbg1 = stdout;
+#else
+   fpdbg1 = stdout;
+#endif
+   if (tag == NULL) stag = "dump_buf"; /*safety valve*/
+   else stag = tag;
+   fprintf(fpdbg1,"%s (len=%d): ", stag,sz);
+   line[0] = 0; line[16] = 0;
+   j = 0;
+   if (sz < 0) { fprintf(fpdbg1,"\n"); return; } /*safety valve*/
+   for (i = 0; i < sz; i++) {
+      if (i % 16 == 0) {
+         line[j] = 0;
+         j = 0;
+         fprintf(fpdbg1,"%s\n  %04x: ",line,i);
+      }
+      if (fshowascii) {
+         a = pbuf[i];
+         if (a < 0x20 || a > 0x7f) a = '.';
+         line[j++] = a;
+      }
+      fprintf(fpdbg1,"%02x ",pbuf[i]);
+   }
+   if (fshowascii) {
+     if ((j > 0) && (j < 16)) {
+       /* space over the remaining number of hex bytes */
+       for (i = 0; i < (16-j); i++) fprintf(fpdbg1,"   ");
+     }
+     else j = 16;
+     line[j] = 0;
+   }
+   fprintf(fpdbg1,"%s\n",line);
+#ifdef DBG_IPMI
+   if (fpdbg1 != stdout) fclose(fpdbg1);
+#endif
+   return;
+}
+
+#ifdef LOCAL_IPMB
+#define SMS_MSG_LUN              0x02
+extern void os_usleep(int s, int u);  /*subs.c*/
+static int sendSeq = 1;
+static BYTE cksum(const BYTE *buf, register int len)
+{     /* 8-bit 2s compliment checksum */
+        register BYTE csum;
+        register int i;
+        csum = 0;
+        for (i = 0; i < len; i++) csum = (csum + buf[i]) % 256;
+        csum = -csum;
+        return(csum);
+}
+
+// SendTimedIpmbpRequest
+ACCESN_STATUS
+SendTimedIpmbpRequest (
+		IMBPREQUESTDATA *reqPtr,         /* request info and data */
+		int     timeOut,         /* how long to wait, in mSec units */
+		BYTE 	*respDataPtr,    /* where to put response data */
+		int 	*respDataLen,    /* how much response data there is */
+		BYTE 	*completionCode  /* request status from bmc controller*/
+	)
+{
+	BYTE                    responseData[MAX_IMB_REQ_SIZE];
+	ImbResponseBuffer *     resp = (ImbResponseBuffer *) responseData;
+	DWORD                   respLength = sizeof( responseData );
+	DWORD                   reqLength;
+	BYTE                    requestData[MAX_IMB_RESP_SIZE];
+	ImbRequestBuffer *      req = (ImbRequestBuffer *) requestData;
+	BOOL                    status;
+	int i;
+
+	req->req.rsSa           =  BMC_SA;
+	req->req.cmd            =  SEND_MESSAGE;	
+	req->req.netFn          =  APP_NETFN;
+	req->req.rsLun          =  BMC_LUN; /*=0*/
+	req->req.data[0] = reqPtr->busType;
+	req->req.data[1] = reqPtr->rsSa;
+	req->req.data[2] = ((reqPtr->netFn << 2) | (reqPtr->rsLun & 0x03));
+	req->req.data[3] = cksum(&req->req.data[1],2);
+	req->req.data[4] = BMC_SA;
+	req->req.data[5] = ((sendSeq << 2) | (SMS_MSG_LUN & 0x03));
+	req->req.data[6] = reqPtr->cmdType;
+	for (i = 0; i < reqPtr->dataLength; i++)
+		req->req.data[7+i] = reqPtr->data[i];
+	req->req.data[7+i] = cksum(&req->req.data[4], reqPtr->dataLength + 3);
+	req->req.dataLength = reqPtr->dataLength + 8;
+
+	req->flags              = 0;
+	req->timeOut    = timeOut * 1000;       /* convert to uSec units */
+	reqLength = req->req.dataLength + MIN_IMB_REQ_BUF_SIZE;
+	status = DeviceIoControl(       hDevice,
+					ioctl_sendmsg,
+					requestData,
+					reqLength,
+					& responseData,
+					sizeof( responseData ),
+					& respLength,
+					NULL);
+	if (fdebug) printf("sendIpmb: send_message status=%d rlen=%d cc=%x\n",
+				status,respLength,resp->cCode);
+	if ( status != TRUE ) {
+		DWORD error;
+		error = GetLastError();
+		return ACCESN_ERROR;
+	}
+	if ( respLength == 0 ) {
+		return ACCESN_ERROR;
+	}
+	sendSeq++;
+	if ( resp->cCode != 0) {
+		*completionCode = resp->cCode;
+		*respDataLen    = 0;
+		return ACCESN_OK;
+	}
+
+	/*
+	 * Sent ok, wait for GetMessage response 
+	 */
+      for (i=0; i < 10; i++)    /* RETRIES=10 for GetMessage  */
+      {
+	//req->req.busType        =  PUBLIC_BUS; /*=0*/
+	req->req.rsSa           =  BMC_SA;
+	req->req.cmd            =  GET_MESSAGE;	
+	req->req.netFn          =  APP_NETFN;
+	req->req.rsLun          =  BMC_LUN; /*=0*/
+	req->req.dataLength = 0;
+	reqLength = req->req.dataLength + MIN_IMB_REQ_BUF_SIZE;
+	status = DeviceIoControl(       hDevice,
+					ioctl_sendmsg,
+					requestData,
+					reqLength,
+					& responseData,
+					sizeof( responseData ),
+					& respLength,
+					NULL);
+	if (fdebug) printf("sendIpmb: get_message status=%d rlen=%d cc=%x\n",
+				status,respLength,resp->cCode);
+	if ( status != TRUE ) {
+		DWORD error;
+		error = GetLastError();
+		return ACCESN_ERROR;
+	}
+	if ( respLength == 0 ) {
+		return ACCESN_ERROR;
+	}
+	if ( (resp->cCode != 0x80) && (resp->cCode != 0x83) ) 
+	   break;  /*success, exit loop*/
+	os_usleep(0,1000);  /* 1 msec*/
+      } /*end-for*/
+
+	/*
+	 * give the caller his response
+	 */
+	*completionCode = resp->cCode;
+	*respDataLen    = 0;
+	if (( respLength > 1 ) && ( respDataPtr)) {
+		/* check that resp cmd & netfn match */
+		*respDataLen    = respLength - 7;
+		memcpy( respDataPtr, &resp->data[7], *respDataLen);
+	}
+	return ACCESN_OK;
+}
+#endif
+
 /*///////////////////////////////////////////////////////////////////////////
 // SendTimedImbpRequest
 ///////////////////////////////////////////////////////////////////////////// */
@@ -319,11 +531,11 @@ SendTimedImbpRequest (
 		BYTE 	*completionCode  /* request status from bmc controller*/
 	)
 {
-	BYTE                    responseData[MAX_RBUFFER_SIZE];
+	BYTE                    responseData[MAX_IMB_REQ_SIZE];
 	ImbResponseBuffer *     resp = (ImbResponseBuffer *) responseData;
 	DWORD                   respLength = sizeof( responseData );
 	DWORD                   reqLength;
-	BYTE                    requestData[MAX_RBUFFER_SIZE];
+	BYTE                    requestData[MAX_IMB_RESP_SIZE];
 	ImbRequestBuffer *      req = (ImbRequestBuffer *) requestData;
 	BOOL                    status;
 
@@ -332,7 +544,7 @@ SendTimedImbpRequest (
 	req->req.cmd            = reqPtr->cmdType;
 	req->req.netFn          = reqPtr->netFn;
 	req->req.rsLun          = reqPtr->rsLun;
-	req->req.dataLength     = reqPtr->dataLength;
+	req->req.dataLength     = (BYTE)reqPtr->dataLength;
 
 #ifndef NO_MACRO_ARGS
 	DEBUG("cmd=%02x, pdata=%p, datalen=%d, size=%d\n", req->req.cmd, 
@@ -365,6 +577,13 @@ SendTimedImbpRequest (
 	printf("%s: rsSa %x cmd %x netFn %x lun %x, status=%d, cc=%x, rlen=%d\n", 
             __FUNCTION__, req->req.rsSa, req->req.cmd, req->req.netFn, 
             req->req.rsLun, status,  resp->cCode, respLength );
+        dbgmsg("SendTimedImb: rsSa %x cmd %x netFn %x lun %x, status=%d, cc=%x, rlen=%d\n", 
+            req->req.rsSa, req->req.cmd, req->req.netFn, 
+            req->req.rsLun, status,  resp->cCode, respLength );
+	if (req->req.cmd == 0x34) {
+          _dump_buf("requestData",requestData,reqLength,0); //if DBG_IPMI
+          _dump_buf("responseData",responseData,respLength,0); //if DBG_IPMI
+        }
 #endif
 
 	if( status != TRUE ) {
@@ -412,14 +631,14 @@ int open_imb(int fskipcmd)
 /* This routine will be called from all other routines before doing any
    interfacing with imb driver. It will open only once. */
 	IMBPREQUESTDATA                requestData;
-	BYTE						   respBuffer[16];
-	DWORD						   respLength;
-	BYTE						   completionCode;
+	BYTE			   respBuffer[MAX_IMB_RESP_SIZE];
+	DWORD			   respLength;
+	BYTE			   completionCode;
+	int ret;
 
+  set_fps();
   if (hDevice1 == 0)  /*INVALID_HANDLE_VALUE*/
   {
-        fpdbg = stdout;
-        fperr = stderr;
         //
         // Open IMB driver device
         //
@@ -432,7 +651,8 @@ int open_imb(int fskipcmd)
                                 NULL
                             );
         if (hDevice == NULL || hDevice == INVALID_HANDLE_VALUE) {
-	    if (fdebug) printf("ipmi_open_ia: error opening %s\n",IMB_DEVICE_WIN);
+	    if (fdebug) printf("ipmi_open_ia: error opening %s to imbdrv.sys\n",
+				IMB_DEVICE_WIN);
             return (0);  /*FALSE*/
 	}
 
@@ -452,15 +672,16 @@ int open_imb(int fskipcmd)
         requestData.busType            = PUBLIC_BUS;
         requestData.data               = NULL;
         requestData.dataLength         = 0;
-        respLength                     = 16;
-        if ( (SendTimedImbpRequest ( &requestData, IMB_OPEN_TIMEOUT,
-                         respBuffer, &respLength, &completionCode
-                        )  != ACCESN_OK ) || ( completionCode != 0) )
+        respLength                     = sizeof(respBuffer);
+	ret = SendTimedImbpRequest ( &requestData, IMB_OPEN_TIMEOUT,
+                         respBuffer, &respLength, &completionCode);
+	if ( (ret != ACCESN_OK ) || ( completionCode != 0) )
         {
-	     if (fdebug) printf("ipmi_open_ia: GetDeviceId error, ccode=%x\n",
-					completionCode);
-             CloseHandle(hDevice);
-             return (0);  /*FALSE*/
+	    if (fdebug)   /*GetDeviceId failed*/
+	        printf("ipmi_open_ia: imbdrv request error, ret=%d ccode=%x\n",
+					ret,completionCode);
+            CloseHandle(hDevice);
+            return (0);  /*FALSE*/
         }
         hDevice1 = hDevice;
 
@@ -484,16 +705,16 @@ int open_imb(int fskipcmd)
 /* This routine will be called from all other routines before doing any
    interfacing with imb driver. It will open only once. */
 	IMBPREQUESTDATA          requestData;
-	BYTE			 respBuffer[16];
+	BYTE			 respBuffer[MAX_IMB_RESP_SIZE];
 	int			 respLength;
 	BYTE			 completionCode;
 
-	int my_ret_code;
+	int ret_code;
+
+  set_fps();
 
   if (hDevice1 == 0)
   {
-      fpdbg = stdout;
-      fperr = stderr;
 #ifndef NO_MACRO_ARGS
 	DEBUG("%s: opening the driver, ioctl_sendmsg = %x\n", 
 		__FUNCTION__, ioctl_sendmsg);
@@ -515,8 +736,6 @@ int open_imb(int fskipcmd)
 		    /*if ((hDevice1 = open(IMB_DEVICE,O_RDWR|O_NDELAY)) <0)  */
 		    if ((hDevice1 = open(IMB_DEVICE,O_RDWR)) <0) 
 			{
-				char buf[128];
-
 				hDevice1  = 0;
 #ifndef WIN32
 				/* dont always display open errors if Linux */
@@ -524,10 +743,8 @@ int open_imb(int fskipcmd)
 #endif
 				{
 				  /*debug or not 1st time */
-				  sprintf(buf,
-				         "%s ipmi_open_ia: open(%s) failed", 
-					 __FILE__,IMB_DEVICE);
-				  perror(buf);
+				  printf("imbapi ipmi_open_ia: open(%s) failed, %s\n", 
+					 IMB_DEVICE,strerror(errno));
 				}
 				return (0);
 			}	
@@ -548,17 +765,18 @@ int open_imb(int fskipcmd)
 			requestData.busType            = PUBLIC_BUS;
 			requestData.data               = NULL; 
 			requestData.dataLength	    = 0;
-			respLength		    = 16;
+			respLength		    = sizeof(respBuffer);
 #ifndef NO_MACRO_ARGS
 			DEBUG("%s: opened driver, getting IPMI version\n", __FUNCTION__);
 #endif
-			if (((my_ret_code = SendTimedImbpRequest(&requestData,
+			ret_code = SendTimedImbpRequest(&requestData,
 				 IMB_OPEN_TIMEOUT, respBuffer, &respLength, 
-				 &completionCode)) != ACCESN_OK ) 
-			    || (completionCode != 0) )
+				 &completionCode);
+			if ( (ret_code != ACCESN_OK) || (completionCode != 0) )
 			{
 			   printf("ipmi_open_ia: SendTimedImbpRequest error. Ret = %d CC = 0x%02X\n",
-					 my_ret_code, completionCode);
+					 ret_code, completionCode);
+			   close(hDevice1);
 			   hDevice1 = 0;
 			   return (0);
 			}
@@ -655,6 +873,18 @@ int ipmi_cmdraw_ia(BYTE cmd, BYTE netfn, BYTE lun, BYTE sa, BYTE bus,
    struct stat     stbuf;
 #endif
 
+   if (fdebug) printf("ipmi_cmdraw_ia(%02x,%02x,%02x,%02x,bus=%02x)\n",
+			cmd,netfn,lun,sa,bus); 
+#ifdef METACOMMAND_IPMB   /*++++ not used*/
+   if (bus != PUBLIC_BUS) {
+	if (fdebug) printf("ipmi_cmdraw_ia: bus=%x, using ipmb\n",bus);
+	status = ipmi_cmd_ipmb(cmd,netfn,lun,sa,bus,pdata,sdata,presp,sresp,
+				pcc,fdebugcmd);
+	return(status);
+   }
+#endif
+   set_fps();
+
    requestData.cmdType	= cmd;
    requestData.rsSa	= sa;
    requestData.busType	= bus;
@@ -692,9 +922,13 @@ int ipmi_cmdraw_ia(BYTE cmd, BYTE netfn, BYTE lun, BYTE sa, BYTE bus,
            for ( i =0 ; i < IMB_MAX_RETRIES; i++) 
 	   {
 		*sresp = sz;   /* retries may need to re-init *sresp */
-		if((status = SendTimedImbpRequest(&requestData, ipmi_timeout_ia, presp, sresp, pcc)) == 0 ) {
-			break;
-			}
+#ifdef LOCAL_IPMB
+		if (bus != PUBLIC_BUS)
+		   status = SendTimedIpmbpRequest(&requestData, ipmi_timeout_ia, presp, sresp, pcc);
+		else 
+#endif
+		   status = SendTimedImbpRequest(&requestData, ipmi_timeout_ia, presp, sresp, pcc);
+		if((status) == 0 ) { break; }
 		if (fdebugcmd)   // only gets here if error
 	          fprintf(fpdbg,"ipmi_cmdraw_ia: sendImbRequest error status=%x, ccode=%x\n",
                             (uint)status, *pcc);
@@ -713,7 +947,7 @@ int ipmi_cmdraw_ia(BYTE cmd, BYTE netfn, BYTE lun, BYTE sa, BYTE bus,
                fprintf(fpdbg,"\n");
             }
         }
-
+    if (status == 1) status = -3;  /*LAN_ERR_RECV_FAIL, a meaningful error*/
     return(status);
 } /* end ipmi_cmdraw_ia() */
 
@@ -921,7 +1155,7 @@ SendTimedEmpMessageResponse (
 		 req->req.data[i] += req->req.data[j+3];
 	req->req.data[i]  = ~(req->req.data[i]) +1;
 	++i;
-	req->req.dataLength     = i;
+	req->req.dataLength     = (BYTE)i;
 
 	req->flags      = 0;
 	req->timeOut    = timeOut * 1000;       /* convert to uSec units */
@@ -1064,7 +1298,7 @@ SendTimedEmpMessageResponse_Ex (
 		 req->req.data[i] += req->req.data[j+3];
 	req->req.data[i]  = ~(req->req.data[i]) +1;
 	++i;
-	req->req.dataLength     = i;
+	req->req.dataLength     = (BYTE)i;
 
 	/*The flags & timeouts are used by the driver internally. */
 	req->flags      = 0;
@@ -1085,7 +1319,12 @@ SendTimedEmpMessageResponse_Ex (
 	DEBUG("%s: DeviceIoControl status = %d\n",__FUNCTION__, status);
 #endif
 
-
+	if (fdebug) {
+		   printf("SendTimedEmp(%x,%x): status=%d cc=%x rlen=%d i=%d\n",
+				sessionHandle, channelNumber,
+				status,responseData[0],respLength,i);
+		   _dump_buf("requestData",requestData,sizeof(requestData),0);
+	}
 	if ( (status != TRUE) || (respLength != 1) || (responseData[0] != 0) )
 	{
 		return ACCESN_ERROR;
@@ -1093,7 +1332,6 @@ SendTimedEmpMessageResponse_Ex (
 	return ACCESN_OK;
 
 
-
 }
 
 /*This is not a  API exported by the driver in stricter sense. It is 
@@ -1172,12 +1410,11 @@ SendTimedLanMessageResponse(
 		 req->req.data[i] += req->req.data[j+3];
 	req->req.data[i]  = ~(req->req.data[i]) +1;
 	++i;
-	req->req.dataLength     = i;
+	req->req.dataLength     = (BYTE)i;
 
 	req->flags      = 0;
 	req->timeOut    = timeOut * 1000;       /* convert to uSec units */
 
-
 	status = DeviceIoControl(       hDevice,
 					ioctl_sendmsg,
 					requestData,
@@ -1188,6 +1425,11 @@ SendTimedLanMessageResponse(
 					NULL
 				  );
 
+	if (fdebug) {
+		   printf("SendTimedLan(): status=%d cc=%x rlen=%d i=%d\n",
+				status, responseData[0],respLength,i);
+		   _dump_buf("requestData",requestData,sizeof(requestData),0);
+	}
 #ifndef NO_MACRO_ARGS
 	DEBUG("%s: DeviceIoControl status = %d\n",__FUNCTION__, status);
 #endif
@@ -1199,6 +1441,7 @@ SendTimedLanMessageResponse(
 	return ACCESN_OK;
 }
 
+
 /*This is not a  API exported by the driver in stricter sense. It is 
 //added to support EMP functionality. Upper level software could have 
 //implemented this function.(pai 5/4/99) */
@@ -1315,13 +1558,12 @@ SendTimedLanMessageResponse_Ex(
 		req->req.data[i] += req->req.data[j+3];
 	req->req.data[i]  = ~(req->req.data[i]) +1;
 	++i;
-	req->req.dataLength     = i;
+	req->req.dataLength     = (BYTE)i;
 
 	/*The flags & timeouts are used by the driver internally */
 	req->flags      = 0;
 	req->timeOut    = timeOut * 1000;       /* convert to uSec units */
 
-
 	status = DeviceIoControl(       hDevice,
 					ioctl_sendmsg,
 					requestData,
@@ -1331,20 +1573,21 @@ SendTimedLanMessageResponse_Ex(
 					& respLength,
 					NULL
 				  );
-
 #ifndef NO_MACRO_ARGS
 	DEBUG("%s: DeviceIoControl status = %d\n",__FUNCTION__, status);
 #endif
 	if (fdebug) {
-	   printf("SendTimedLan: status=%d cc=%x rlen=%d\n",
-				status,responseData[0],respLength);
-	   if (responseData[0] == 0xcc) { /*invalid request data*/
+	   printf("SendTimedLan(%x,%x): status=%d cc=%x rlen=%d i=%d\n",
+				sessionHandle, channelNumber,
+				status,responseData[0],respLength,i);
+	   if (responseData[0] != 0)  /*0xcc == invalid request data*/
+	   {
+	      BYTE *pb;
+	      pb = (BYTE *)req->req.data;
 	      printf("SendMessage data: %02x %02x %02x %02x %02x %02x %02x %02x\n",
-	         req->req.data[0], req->req.data[1], req->req.data[2],
-	         req->req.data[3], req->req.data[4], req->req.data[5],
-	         req->req.data[6], req->req.data[7]);
+		 pb[0], pb[1], pb[2], pb[3], pb[4], pb[5], pb[6], pb[7]);
+	      _dump_buf("requestData",requestData,sizeof(requestData),0);
 	   }
-
 	}
 
 	if ( (status != TRUE) || (respLength != 1) || (responseData[0] != 0) )
@@ -1387,7 +1630,7 @@ SendAsyncImbpRequest (
 	req->req.cmd            = reqPtr->cmdType;
 	req->req.netFn          = reqPtr->netFn;
 	req->req.rsLun          = reqPtr->rsLun;
-	req->req.dataLength     = reqPtr->dataLength;
+	req->req.dataLength     = (BYTE)reqPtr->dataLength;
 
 	memcpy( req->req.data, reqPtr->data, reqPtr->dataLength );
 
@@ -1764,7 +2007,7 @@ GetAsyncImbpMessage_Ex (
 //
 *F*/
 ACCESN_STATUS
-IsAsyncMessageAvailable (unsigned int   eventId )
+IsAsyncMessageAvailable (HandleType   eventId )
 {
     int 	dummy;
     int 	respLength = 0;
@@ -1773,14 +2016,18 @@ IsAsyncMessageAvailable (unsigned int   eventId )
  /* confirm that app is not using a bad Id */
 
 
-	if (  AsyncEventHandle  != (HANDLE) eventId)
+	if ( AsyncEventHandle  != eventId) {
+#ifdef LINUX
+		printf("Invalid AsyncHandle %x!=%x\n",AsyncEventHandle,eventId);
+#endif
 	  	return ACCESN_ERROR;
+	}
 
 	dummy = 0;
 	status = DeviceIoControl(hDevice,
 				   IOCTL_IMB_CHECK_EVENT,
 				   &AsyncEventHandle,
-				    sizeof(HANDLE ),
+				    sizeof(HandleType),
 				    &dummy,
 					sizeof(int),
 					(LPDWORD) & respLength,
@@ -1962,7 +2209,7 @@ GetAsyncDpcMessage (
 //              get the Async messages..
 *F*/
 ACCESN_STATUS
-RegisterForImbAsyncMessageNotification (unsigned int *handleId)
+RegisterForImbAsyncMessageNotification (HandleType *handleId)
 
 {
 	BOOL      status;
@@ -1980,18 +2227,24 @@ RegisterForImbAsyncMessageNotification (unsigned int *handleId)
 				&dummy,
 				sizeof( int ),
 				&AsyncEventHandle,
-				(DWORD)sizeof(HANDLE ),
+				sizeof(HandleType),
 				(LPDWORD) & respLength,
 				NULL
 			  );
 #ifndef NO_MACRO_ARGS
 	DEBUG("%s: DeviceIoControl status = %d\n",__FUNCTION__, status);
 #endif
-
-	if( (respLength != sizeof(int))  || (status != TRUE ))
+	
+	if( (respLength != sizeof(int))  || (status != TRUE )) {
+		if (fdebug) {
+		   printf("RegisterForImbAsync error status=%d, len=%d sizeint=%d\n", status, respLength, sizeof(int));
+		  if( respLength != sizeof(int)) printf("Async len err\n");
+	 	  if( status != TRUE) printf("Async status err\n");
+		}
 		return  ACCESN_ERROR;
+	}
 
-	*handleId = (unsigned int) AsyncEventHandle;
+	*handleId = AsyncEventHandle;
 	
 #ifndef NO_MACRO_ARGS
 	DEBUG("handleId = %x AsyncEventHandle %x\n", *handleId, AsyncEventHandle);
@@ -2023,7 +2276,7 @@ RegisterForImbAsyncMessageNotification (unsigned int *handleId)
 //  Notes:      
 *F*/
 ACCESN_STATUS
-UnRegisterForImbAsyncMessageNotification (unsigned int handleId, int iFlag)
+UnRegisterForImbAsyncMessageNotification (HandleType handleId, int iFlag)
 
 {
 	BOOL		status;
@@ -2032,13 +2285,13 @@ UnRegisterForImbAsyncMessageNotification (unsigned int handleId, int iFlag)
 
 	iFlag = iFlag;	/* to keep compiler happy  We are not using this flag*/
 
-	if (  AsyncEventHandle  != (HANDLE) handleId)
+	if ( AsyncEventHandle  != handleId)
 	  return ACCESN_ERROR;
 
 	status = DeviceIoControl(hDevice,
 					IOCTL_IMB_DEREGISTER_ASYNC_OBJ,
 					&AsyncEventHandle,
-					(DWORD)sizeof(HANDLE ),
+					sizeof(HandleType),
 					&dummy,
 					(DWORD)sizeof(int ),
 					(LPDWORD) & respLength,
@@ -2315,11 +2568,11 @@ UnmapPhysicalMemory( int virtualAddress, int Length )
 *F*/
 BYTE	GetIpmiVersion()
 {
-	return	IpmiVersion;
+	return	((BYTE)IpmiVersion);
 }
 
-#ifdef WIN32
-/* Inlude this routine if building imbapi.dll */
+#ifdef IMBDLL
+/* Inlude this routine if building WIN32 imbapi.dll */
 BOOL WINAPI DllMain(HINSTANCE hInstance, ULONG ulReason, LPVOID pvReserved)
 {
    switch(ulReason)
@@ -2335,5 +2588,7 @@ BOOL WINAPI DllMain(HINSTANCE hInstance, ULONG ulReason, LPVOID pvReserved)
    }
    return TRUE;
 }
+#endif
+
 #endif
 /*end of imbapi.c*/
